import java.io.IOException;
import java.util.HashMap;

/**
 * This class simulates a search engine for a Dictionary of antonyms and
 * synonyms using a HashMap<word, Entry> as its underlying data structure
 * 
 * @author Sandino Vargas-Perez
 * @version 31 May 2020
 *
 */
public class SearchEngine1 extends Engine {

	private HashMap<String, DictionaryEntry> dictionary;

	public SearchEngine1(String filename) {
		super(filename);
		this.dictionary = new HashMap<String, DictionaryEntry>(20000);
		populateDictionary();
		//System.out.println("Size: "+dictionary.size());
	}

	@Override
	protected void populateDictionary() {
		// Read in each line, parse into words, and add words to the map.
		String nextLine = null;
		DictionaryEntry nymsList = null;
		try {
			// Read lines until hitting a null or blank line.
			while (((nextLine = reader.readLine()) != null)) {
				// In the file and "=" represents the end of an entry
				if (nextLine.equals("=")) {
					dictionary.put(nymsList.getWord(), nymsList);
					continue;
				}

				if (nextLine.contains("KEY")) {
					nymsList = new DictionaryEntry(nextLine.replace("KEY:", "").toLowerCase());

				} else if (nextLine.contains("SYN")) {
					nymsList.setSynonyms(nextLine.replace("SYN:", "").toLowerCase());
				} else if (nextLine.contains("ANT")) {
					nymsList.setAntonyms(nextLine.replace("ANT:", "").toLowerCase());
				}

			}
		} catch (IOException e) {
			System.err.println("Could not read file " + filename);
		}

	}

	@Override
	public void search(String word) {
		long startTime, duration;
		word = word.toLowerCase();
		
		// Start timer
		startTime = System.nanoTime();

		DictionaryEntry entry = dictionary.get(word);
		
		duration = (System.nanoTime() - startTime);
		System.out.println("Engine 1 took: " + duration + " nsecs to find " + word);
		// End timer
		
		// If the word is in the dictionary, display its syns. and ants.
		// Display a "No Found" if the dictionary does not contain the word.
		if (entry != null) {
			display.displayResults(entry);
		} else
			display.displayNotFound(word);
	}

	@Override
	public boolean contains(String word) {
		return dictionary.containsKey(word);
	}

}
