import java.io.IOException;
import java.util.ArrayList;

/**
 * This class simulates a search engine for a Dictionary of antonyms and
 * synonyms using an ArrayList<Entries> as its underlying data structure
 * 
 * @author Sandino Vargas-Perez
 * @version 31 May 2020
 *
 */
public class SearchEngine2 extends Engine {
	private ArrayList<DictionaryEntry> dictionary;
	int index;

	public SearchEngine2(String filename) {
		super(filename);
		this.dictionary = new ArrayList<DictionaryEntry>();
		populateDictionary();
		this.index = -1;

	}

	@Override
	protected void populateDictionary() {
		// Read in each line, parse into words, and add words to the map.
		String nextLine = null;
		DictionaryEntry nymsList = null;
		try {
			// Read lines until hitting a null or blank line.
			while (((nextLine = reader.readLine()) != null)) {
				if (nextLine.equals("=")) {
					dictionary.add(nymsList);
					continue;
				}

				if (nextLine.contains("KEY")) {
					nymsList = new DictionaryEntry(nextLine.replace("KEY:", "").toLowerCase());

				} else if (nextLine.contains("SYN")) {
					nymsList.setSynonyms(nextLine.replace("SYN:", "").toLowerCase());
				} else if (nextLine.contains("ANT")) {
					nymsList.setAntonyms(nextLine.replace("ANT:", "").toLowerCase());
				}

			}
		} catch (IOException e) {
			System.err.println("Could not read file " + filename);
		}

	}

	@Override
	public void search(String word) {
		long startTime, duration;
		word = word.toLowerCase();

		// Start timer
		startTime = System.nanoTime();

		DictionaryEntry entry = null;

		if (contains(word))
			entry = dictionary.get(index);

		duration = (System.nanoTime() - startTime);
		System.out.println("Engine 2 took: " + duration + " nsecs to find " + word);
		// End timer
		
		// Display results
		if (entry != null) {
			display.displayResults(entry);
		} else
			display.displayNotFound(word);
	}

	@Override
	public boolean contains(String word) {
		for (DictionaryEntry entry : dictionary) {
			if (entry.getWord().equals(word)) {
				index = dictionary.indexOf(entry);
				return true;
			}
		}
		return false;
	}

}
